import matplotlib.pyplot as plt

''' 
The following functions (Susceptible, Infected, Recovered) represent S'(t), I'(t) and
R'(t) of the SIR model. These are numerically solved using the 4th-order Runge-Kutta model.
'''
def Susceptible(s, i):
    # S'(t) = -beta * S(t) * I(t)
    St = -b * s * i
    return St

def Infected(s, i):
    # I'(t) = beta * S(t) * I(t) - gamma * I(t)
    It = (b * s * i) - (g * i)
    return It

def Recovered(i):
    # R'(t) = gamma * I(t)
    Rt = g * i
    return Rt

'''
Variables and Lists
'''
b = 0.00000005524411621 # beta median value (obtained using finite difference method).
g = 0.09727437916 # gamma median value (obtained using finite difference method).
days = 900 # Number of days we want to forecast.
h = 0.05 # Step size of the RK4 method.
t = range(int(days/h)) # Will be used in plot.

Sus = [0] * int(days / h) # We define a list of length days/h to save all susceptible iterations.
Inf = [0] * int(days / h) # We define a list of length days/h to save all infected iterations.
Rec = [0] * int(days / h) # We define a list of length days/h to save all recovered iterations.

Sus[0] = 2279950 # Initial susceptible people (Population in Queretaro).
Inf[0] = 22 # Initial infected people.
Rec[0] = 18 # Initial recovered people.

'''
Solving the SIR model using RK4
'''
for n in range(int(days/h) - 1):
    ''' 
    First, we iterate Susceptible
    The following variables (k1, ..., k4) represent the slopes used to approximate the 
    solution of the differential equation in discrete steps.
    '''
    k1 = Susceptible(Sus[n], Inf[n])
    k2 = Susceptible(Sus[n] + (1/2 * h * k1), Inf[n])
    k3 = Susceptible(Sus[n] + (1/2 * h * k2), Inf[n])
    k4 = Susceptible(Sus[n] + (h * k3), Inf[n])
    Sus[n + 1] = Sus[n] + ((h/6) * (k1 + 2*k2 + 2*k3 + k4))
    ''' 
    Second, we iterate Infected
    '''
    k1 = Infected(Sus[n], Inf[n])
    k2 = Infected(Sus[n], Inf[n]  + (1/2 * h * k1))
    k3 = Infected(Sus[n], Inf[n]  + (1/2 * h * k2))
    k4 = Infected(Sus[n], Inf[n]  + (h * k3))
    Inf[n + 1] = Inf[n] + ((h/6) * (k1 + 2*k2 + 2*k3 + k4))
    ''' 
    Third, we iterate Recovered
    '''
    k1 = Recovered(Inf[n])
    k2 = Recovered(Inf[n]  + (1/2 * h * k1))
    k3 = Recovered(Inf[n]  + (1/2 * h * k2))
    k4 = Recovered(Inf[n]  + (h * k3))
    Rec[n + 1] = Rec[n] + ((h/6) * (k1 + 2*k2 + 2*k3 + k4))

# Plot SIR model
plt.plot(t, Sus, label = "Susceptible", color = 'blue')
plt.plot(t, Inf, label = "Infected", color = 'red')
plt.plot(t, Rec, label = "Recovered", color = 'green')

# Plot decoration
plt.title('SIR model in Queretaro')
plt.xlabel('Day / h')
plt.ylabel('Millions of People')
plt.legend()

plt.grid()
plt.show()